@@ -7,10 +7,11 @@ use Data::Transform::ExplicitMetadata qw(encode);
 
 use base 'Devel::hdb::App::Base';
 
-use Devel::hdb::Response;
+use Devel::hdb::Utils;
 
 __PACKAGE__->add_route('post', '/eval', \&do_eval);
-__PACKAGE__->add_route('post', '/getvar', \&do_getvar);
+__PACKAGE__->add_route('get', qr{/getvar/(\d+)/([^/]+)}, \&do_getvar);
+__PACKAGE__->add_route('get', qr{/getvar/(\d+)}, \&list_vars_at_level);
 
 # Evaluate some expression in the debugged program's context.
 # It works because when string-eval is used, and it's run from
@@ -22,18 +23,11 @@ __PACKAGE__->add_route('post', '/getvar', \&do_getvar);
 sub do_eval {
     my($class, $app, $env) = @_;
 
-    my $req = Plack::Request->new($env);
-    my $expr = $req->content();
-    my $eval_string = _fixup_expr_for_eval($expr);
+    my $body = $class->_read_request_body($env);
+    my $params = $app->decode_json($body);
+    my $eval_string = Devel::hdb::Utils::_fixup_expr_for_eval($params->{code});
 
-    my $resp = Devel::hdb::Response->new('evalresult', $env);
-
-    my $result_packager = sub {
-        my $data = shift;
-        $data->{expr} = $expr;
-        return $data;
-    };
-    return _eval_plumbing_closure($app, $eval_string,$resp, $env, $result_packager);
+    return _eval_plumbing_closure($app, $env, $eval_string, $params->{wantarray});
 }
 
 my %perl_special_vars = map { $_ => 1 }
@@ -51,91 +45,61 @@ $perl_special_vars{q{$)}} = 1;
 
 # Get the value of a variable, possibly in an upper stack frame
 sub do_getvar {
-    my($class, $app, $env) = @_;
-
-    my $req = Plack::Request->new($env);
-    my $level = $req->param('l');
-    my $varname = $req->param('v');
-
-    my $resp = Devel::hdb::Response->new('getvar', $env);
+    my($class, $app, $env, $level, $varname) = @_;
 
     if ($perl_special_vars{$varname}) {
-        my $result_packager = sub {
-            my $data = shift;
-            $data->{expr} = $varname;
-            $data->{level} = $level;
-            return $data;
-        };
-        return _eval_plumbing_closure($app,$varname, $resp, $env, $result_packager);
+        my $wantarray = substr($varname, 0, 1) eq '$' ? 0 : 1;
+        return _eval_plumbing_closure($app, $env, $varname, $wantarray);
     }
 
     my $value = eval { $app->get_var_at_level($varname, $level) };
     my $exception = $@;
 
-    my $resp_data = { expr => $varname, level => $level };
     if ($exception) {
         if ($exception =~ m/Can't locate PadWalker/) {
-            $resp->{type} = 'error';
-            $resp->data('Not implemented - PadWalker module is not available');
+            return [ 501,
+                    [ 'Content-Type' => 'text/html'],
+                    [ 'Not implemented - PadWalker module is not available'] ];
 
         } elsif ($exception =~ m/Not nested deeply enough/) {
-            $resp_data->{result} = undef;
+            return [ 404,
+                    [ 'Content-Type' => 'text/html' ],
+                    [ 'Stack level not found' ] ];
         } else {
             die $exception
         }
-    } else {
-        $value = encode($value);
-        $resp_data->{result} = $value;
     }
-    $resp->data($resp_data);
+
+    my $result = Data::Transform::ExplicitMetadata::encode($value);
     return [ 200,
             [ 'Content-Type' => 'application/json' ],
-            [ $resp->encode() ]
+            [ $app->encode_json($result) ]
         ];
 }
 
 sub _eval_plumbing_closure {
-    my($app, $eval_string, $resp, $env, $result_packager) = @_;
+    my($app, $env, $eval_string, $wantarray) = @_;
 
-    $eval_string = _fixup_expr_for_eval($eval_string);
+    $eval_string = Devel::hdb::Utils::_fixup_expr_for_eval($eval_string);
     return sub {
         my $responder = shift;
-        my $writer = $responder->([ 200, [ 'Content-Type' => 'application/json' ]]);
         $env->{'psgix.harakiri.commit'} = Plack::Util::TRUE;
 
         $app->eval(
             $eval_string,
-            1,
+            $wantarray,
             sub {
-                my($result, $exception) = @_;
-                my $data;
-                if ($exception) {
-                    $data->{exception} = encode($exception);
-                } else {
-                    $data->{result} = encode(@$result < 2 ? $result->[0] : $result );
-                }
-                $data = $result_packager->($data);
+                my($eval_result, $exception) = @_;
 
-                $resp->data($data);
-                $writer->write($resp->encode());
-                $writer->close();
+                my $result = Data::Transform::ExplicitMetadata::encode($exception || $eval_result);
+                $responder->([ $exception ? 409 : 200,
+                                [ 'Content-Type' => 'application/json' ],
+                                [ $app->encode_json($result) ]]);
             }
         );
     };
 }
 
-# This substitution is done so that we return HASH, as opposed to a list
-# An expression of %hash results in a list of key/value pairs that can't
-# be distinguished from a list.  A glob gets replaced by a glob ref.
-sub _fixup_expr_for_eval {
-    my($expr) = @_;
-
-    $expr =~ s/^\s*(?<!\\)([%*])/\\$1/o;
-    return $expr;
-}
-
-
-
 1;
 
 =pod
@@ -156,16 +120,30 @@ variables in the debugged program.
 =item POST /eval
 
 Evaluate a string of Perl code in the context of the debugged process.
-The Perl code to evaluate is in the body of the POST request.
-
 The code is evaluated in the content of the nearest stack frame that
-is not part of the debugger.
+is not part of the debugger.  The request body must contain a JSON-encoded
+hash with these keys:
 
-=item POST /getvar
+  code      => String of Perl code to evaluate
+  wantarray => 0, 1 or undef; whether to evaluate the code in scalar list
+               or void context
 
-This route requires 2 parameters:
-  l    Number of stack frames above the current one
-  v    Name of the variable, including the sigil
+Returns 200 if successful and the result in the body.  The body contents
+should be decoded using Data::Transform::ExplicitMetadata
+Returns 409 if there was an exception.  The body contents should be decoded
+using Data::Transform::ExplicitMetadata
+
+=item GET /getvar/<level>
+
+Get a list of all the lexical variables at the given stack level.
+Return a JSON-encoded array containing hashes with these keys:
+
+  name => Name of the variable, including the sigil
+  href => URL to use to get the value of the variable
+
+Returns 404 if the requested stack level does not exist.
+
+=item GET /getvar/<level>/<varname>
 
 Searches the requested stack frame for the named variable.  0 is the currently
 executing stack frame, 1 is the frame above that, etc.  The variable must
@@ -179,6 +157,10 @@ portion of a composite value.  For example:
   @hash{key1,key2}  Hash slice
   @array[1 .. 2]    Array slice with a range
 
+Returns 200 and JSON in the body.  The returned JSON is an
+encoded version of whatever the Perl code evaluated to, and should
+be decoded with Data::Transform::ExplicitMetadata.
+
 =back
 
 =head1 SEE ALSO
