@@ -13,7 +13,7 @@ use Scalar::Util qw(reftype);
 
 use Devel::hdb::Utils;
 
-our $VERSION = '0.25';
+our $VERSION = "1.0";
 
 use Exception::Class (
         'Devel::hdb::Client::Exception',
@@ -124,95 +124,52 @@ sub gui {
     return $response->content;
 }
 
-sub _deserialize_status {
-    my $status = shift;
-    if ($status->{events}
-        and
-        my @watchpoint_events = grep { $_->{type} eq 'watchpoint' } @{$status->{events}}
-    ) {
-        foreach my $event ( @watchpoint_events ) {
-            $event->{$_} = Data::Transform::ExplicitMetadata::decode($event->{$_}) foreach (qw(old new));
-        }
-    }
-    return $status;
-}
-
-sub _encode_url {
-    my $base = shift;
-    my @params;
-    for(my $i = 0; $i < @_; $i += 2) {
-        if (defined $_[$i + 1]) {
-            push @params, join('=', @_[$i, $i+1])
-        } else {
-            push @params, $_[$i];
-        }
-    }
-    return join('', $base, '?', join('&', @params));
-}
-
-sub _validate_params {
-    my $params = shift;
-    my %allowed = map { $_ => 1 } @_;
-    for (my $i = 0; $i < @$params; $i += 2) {
-        unless (exists $allowed{ $params->[$i] }) {
-            Carp::croak('Unrecognized param ' . $params->[$i]);
-        }
-    }
-    return 1;
-}
-
 sub stepin {
     my $self = shift;
 
-    _validate_params(\@_, qw(next_statement next_fragment));
-    my $url = _encode_url('stepin', @_);
-    my $response = $self->_POST($url);
+    my $response = $self->_POST('stepin');
     _assert_success($response, q(Can't stepin));
-    return _deserialize_status $JSON->decode($response->content);
+    return $JSON->decode($response->content);
 }
 
 sub stepover {
     my $self = shift;
 
-    _validate_params(\@_, qw(next_statement next_fragment));
-    my $url = _encode_url('stepover', @_);
-    my $response = $self->_POST($url);
+    my $response = $self->_POST('stepover');
     _assert_success($response, q(Can't stepover));
-    return _deserialize_status $JSON->decode($response->content);
+    return $JSON->decode($response->content);
 }
 
 sub stepout {
     my $self = shift;
 
-    _validate_params(\@_, qw(next_statement next_fragment));
-    my $url = _encode_url('stepout', @_);
-    my $response = $self->_POST($url);
+    my $response = $self->_POST('stepout');
     _assert_success($response, q(Can't stepover));
-    return _deserialize_status $JSON->decode($response->content);
+    return $JSON->decode($response->content);
 }
 
 sub continue {
     my $self = shift;
+    my $nostop = shift;
 
-    _validate_params(\@_, qw(nostop next_statement next_fragment));
-    my %params = @_;
-    my $url = _encode_url('continue', %params);
+    my $url = 'continue';
+    if ($nostop) {
+        $url .= '?nostop=1';
+    }
 
     my $response = $self->_POST($url);
     _assert_success($response, q(Can't continue'));
-    return $params{nostop}
+    return $nostop
                 ? 1
-                : _deserialize_status $JSON->decode($response->content);
+                : $JSON->decode($response->content);
 }
 
 sub status {
     my $self = shift;
 
-    _validate_params(\@_, qw(next_statement next_fragment));
-    my $url = _encode_url('status', @_);
-    my $response = $self->_GET($url);
+    my $response = $self->_GET('status');
     _assert_success($response, q(Can't get status));
-    return _deserialize_status $JSON->decode($response->content);
+    return $JSON->decode($response->content);
 }
 
 sub overview {
@@ -258,38 +215,34 @@ my $create_action = "create_action";
 
 foreach my $type ( qw(breakpoint action) ) {
     # change_breakpoint() and change_action()
-    my $change_subname = "change_$type";
     my $change = sub {
         my($self, $bp, %params) = @_;
 
         my $response = $self->_POST($bp, \%params);
-        _assert_success($response, "Can't $change_subname");
+        _assert_success($response, "Can't change $type");
         return $JSON->decode($response->content);
     };
 
     # delete_breakpoint() and delete_action()
-    my $delete_subname = "delete_$type";
     my $delete = sub {
         my($self, $href) = @_;
 
         my $response = $self->_DELETE($href);
-        _assert_success($response, "Can't $delete_subname");
+        _assert_success($response, "Can't delete $type");
         return 1;
     };
 
     # get_breakpoint() and get_action()
-    my $get_one_subname = "get_$type";
     my $get_one = sub {
         my($self, $href) = @_;
 
         my $response = $self->_GET($href);
-        _assert_success($response, "Can't $get_one_subname");
+        _assert_success($response, "Can't get $type");
 
         my $bp = $JSON->decode($response->content);
         return $bp;
     };
 
-    my $get_multiple_subname = "get_${type}s";
     my $get_multiple = do {
         my @recognized_params = qw(filename line code inactive);
 
@@ -304,12 +257,17 @@ foreach my $type ( qw(breakpoint action) ) {
             my $query_string = _encode_query_string_for_hash(%filters);
             $url .= '?' . $query_string if length($query_string);
             my $response = $self->_GET($url);
-            _assert_success($response, "Can't $get_multiple_subname");
+            _assert_success($response, "Can't get $type");
 
             return $JSON->decode($response->content);
         };
     };
 
+    my $change_subname = "change_$type";
+    my $delete_subname = "delete_$type";
+    my $get_one_subname = "get_$type";
+    my $get_multiple_subname = "get_${type}s";
+
     no strict 'refs';
     *$change_subname = $change;
     *$delete_subname = $delete;
@@ -415,14 +373,14 @@ sub load_config {
     my $response = $self->_POST(join('/', 'loadconfig', $escaped_filename));
     _assert_success($response, "Loading config from $filename failed: " . $response->content);
 
-    return $JSON->decode($response->content);
+    return 1;
 }
 
 sub save_config {
-    my($self, $filename, $additional) = @_;
+    my($self, $filename) = @_;
 
     my $escaped_filename = URI::Escape::uri_escape($filename);
-    my $response = $self->_POST(join('/', 'saveconfig', $escaped_filename), $additional);
+    my $response = $self->_POST(join('/', 'saveconfig', $escaped_filename));
     _assert_success($response, "Save config to $filename failed: " . $response->content);
 
     return 1;
@@ -457,41 +415,6 @@ sub sub_info {
     return $JSON->decode($response->content);
 }
 
-sub add_watchpoint {
-    my($self, $expr) = @_;
-
-    my $escaped_expr = URI::Escape::uri_escape($expr);
-    my $response = $self->_PUT(join('/', 'watchpoints', $escaped_expr));
-    _assert_success($response, "Cannot add watchpoint for $expr");
-
-    return 1;
-}
-
-sub delete_watchpoint {
-    my($self, $expr) = @_;
-
-    my $escaped_expr = URI::Escape::uri_escape($expr);
-    my $response = $self->_DELETE(join('/', 'watchpoints', $escaped_expr));
-    _assert_success($response, "Cannot delete watchpoint for $expr");
-
-    return 1;
-}
-
-sub get_watchpoints {
-    my $self = shift;
-
-    my $response = $self->_GET('watchpoints');
-    _assert_success($response, 'Cannot get watchpoints');
-
-    return $JSON->decode($response->content);
-}
-
-sub print_optree {
-    my $self = shift;
-    my $response = $self->_GET('print_optree');
-    _assert_success($response, 'Cannot print_optree');
-}
-
 sub _encode_query_string_for_hash {
     my @params;
     for(my $i = 0; $i < @_; $i += 2) {
@@ -579,11 +502,6 @@ sub _POST {
     $self->_http_request('POST', @_);
 }
 
-sub _PUT {
-    my $self = shift;
-    $self->_http_request('PUT', @_);
-}
-
 sub _HEAD {
     my $self = shift;
     $self->_http_request('HEAD', @_);
@@ -890,35 +808,6 @@ then it returns all breakpoints or actions.
 
 The return value is a listref of hashrefs.
 
-=item $client->add_watchpoint($expression)
-
-Add a watchpoint expression.  These expressions are evaluated before each
-statement in the program.  If their value ever changes, the program will
-stop and the status will include a 'watchpoint' event indicating which line
-caused the change.
-
-=item $client->delete_watchpoint($expression)
-
-Remove a watchpoint expression.  It must have been previously added with
-C<add_watchpoint()> or an exception will be thrown.
-
-=item $client->get_watchpoints($expression)
-
-Return a listref of hashrefs with all the currently set watchpoints.  Each
-hashref has these keys
-
-=over 2
-
-=item expr
-
-The watchpoint expression
-
-=item href
-
-A URL uniquely identifying this watchpoint
-
-=back
-
 =item $client->loaded_files()
 
 Perform GET /source
@@ -1060,38 +949,6 @@ URL to POST to tell the child to run without stopping.
 
 =back
 
-=head2 watchpoint event
-
-When a watchpoint expression's value changes.
-
-=over 2
-
-=item expr
-
-The perl expression whose value changed
-
-=item old
-
-The old value of the expression.  Watchpoint expressions are evaluated in
-list context, so old will always be a listref.
-
-=item new
-
-The new value of the expression.  Also a listref.
-
-=item filename
-
-=item line
-
-=item package
-
-=item subroutine
-
-The location where the change likely happened.  This is whichever line was
-executing immediately before the change was detected.
-
-=back
-
 =head2 exception event
 
 When the program throws an uncaught exception.
@@ -1121,7 +978,7 @@ When the debugged program has finished.  The debugger is still running.
 
 =over 2
 
-=item value
+=item exit_code
 
 The process exit code
 
@@ -1193,7 +1050,7 @@ and content from the response.
 
 =head1 SEE ALSO
 
-L<Devel::hdb>, L<Data::Transform::ExplicitMetadata>
+Devel::hdb, Data::Transform::ExplicitMetadata
 
 =head1 AUTHOR
 
@@ -1201,5 +1058,5 @@ Anthony Brummett <brummett@cpan.org>
 
 =head1 COPYRIGHT
 
-Copyright 2018, Anthony Brummett.  This module is free software. It may
+Copyright 2014, Anthony Brummett.  This module is free software. It may
 be used, redistributed and/or modified under the same terms as Perl itself.
