@@ -1,10 +1,5 @@
 package Devel::hdb::Utils;
 
-use strict;
-use warnings;
-
-our $VERSION = '0.23_15';
-
 # This substitution is done so that we return HASH, as opposed to a list
 # An expression of %hash results in a list of key/value pairs that can't
 # be distinguished from a list.  A glob gets replaced by a glob ref.
