@@ -1,7 +1,6 @@
 function Debugger(sel) {
     var dbg = this;
     var $elt = this.$elt = $(sel);
-    var perlVarPopoverDelay = 400;  // Time in ms the mouse has to rest over a variable before it gets the value
 
     this.filewindowDiv = $elt.find('div#filewindow');
     this.stackDiv = $elt.find('div#stack');
@@ -26,7 +25,6 @@ function Debugger(sel) {
         breakpointRightClickMenu: Handlebars.compile( $('#breakpoint-right-click-template').html() ),
         saveLoadBreakpointsModal: Handlebars.compile($('#save-load-breakpoints-modal-template').html() ),
         subPickerTemplate: Handlebars.compile($('#sub-picker-template').html() ),
-        quickBreakpointModal: Handlebars.compile($('#quick-breakpoint-entry-template').html() ),
     };
 
     // The step in, over, run buttons
@@ -127,8 +125,8 @@ function Debugger(sel) {
                         is_string_eval: is_string_eval,
                         wantarray: frame_obj.sigil,
                         active: $tabs.children().length == 0
-                    }))
-                    .tooltip();
+                    }));
+            var $tab_link = $tab.find('a[rel="tooltip"]').tooltip();
 
             $codePane = $( this.templates.navPane({
                                 serial: frame_obj.serial,
@@ -142,7 +140,7 @@ function Debugger(sel) {
             setElementHeight.call($codePane.find('.managed-height'));
 
             var needs_args = 1;
-            $tab.find('a').on('click', function() {
+            $tab_link.on('click', function() {
                 if (needs_args) {
                     needs_args = 0;
 
@@ -267,11 +265,10 @@ function Debugger(sel) {
     // This function is called after each control button has returned control
     // to us (step, run, etc).  The stack tabs will already be updated, but
     // enything else that needs updated goes in here.
-    function done_after_stack_update(next_statement) {
+    function done_after_stack_update() {
         $control_buttons.attr('disabled', false);
         $('#stack-tabs a:first').trigger('click');
         this.watchedExpressionManager.updateExpressions();
-        this.setCurrentStatementForCodeTable(next_statement);
     }
 
     function notifyProgramHasCompletelyExited() {
@@ -282,56 +279,6 @@ function Debugger(sel) {
         alert('Debugged program has exited');
     }
 
-    var $original_line_elt;
-    var original_line_of_code = '';
-    this.setCurrentStatementForCodeTable = function(next_statement) {
-        var $topLevelCodePane = $('div.tab-pane[data-frameno=0]'),
-            $line = $topLevelCodePane.find('.code-line.active span.code'),
-            stackFrame = this.stackManager.frame(0),
-            filename = stackFrame.filename,
-            lineno = stackFrame.line,
-            this_line_code = $line.text();
-
-        if ($original_line_elt) {
-            $original_line_elt.html(original_line_of_code);
-            $original_line_elt = undefined;
-        }
-
-        if (next_statement) {
-            // Basic chars that need to be escaped (but not parens)
-            var next_statement_re = next_statement.replace(/([.*+?^=!:${}|\[\]\/\\])/g, "\\$1");
-
-            // The deparsed next_statement can have whitespace differences from the original
-            next_statement_re = next_statement_re.replace(/\s+/g, '\\s*');
-
-            // parens can also be optional, maybe with whitespace before/after
-            next_statement_re = next_statement_re.replace(/\(/g, '\\s*\\(?\\s*');
-            next_statement_re = next_statement_re.replace(/\)/g, '\\s*\\)?\\s*');
-
-            // qq(...) is the same as "..."
-            next_statement_re = next_statement_re.replace(/(?:"|qq\\s\*\\\()(.*)(?:"|\\\))/, function(quoted, str) {
-                                                            return '(?:"|qq\\s*\\(?:\(|\{|\[|\/))' + str + '(?:"|qq\\(?:\)|\}|\]|\/))';
-                                                        });
-
-            next_statement_re = RegExp(next_statement_re);
-            // Wrap a <span> around the next_statement
-            var matched = false;
-            var marked_up_this_line_code = this_line_code.replace(next_statement_re, function (code) {
-                                                matched = true;
-                                                return '<span class="next-statement">' + code + '</span>' });
-            if (matched) {
-                // save the original look of the current line so we can restore it next time we're
-                // back in this function
-                $original_line_elt = $line;
-                original_line_of_code = $line.html();
-                // Write the marked up line back into the window
-                $line.html(marked_up_this_line_code);
-            } else {
-                console.warn("didn't find next_statement on line " + lineno + ": " + next_statement);
-            }
-        }
-    };
-
     this.controlButtonClicked = function(e) {
         e.preventDefault();
 
@@ -354,17 +301,16 @@ function Debugger(sel) {
     };
 
     this.handleControlButtonResponse = function(response) {
-        var events = [],
-            watchedExpressionManager = this.watchedExpressionManager;
+        var events = [];
         if (response.events) {
             events = response.events.map(function(event) {
-                return new ProgramEvent(event, watchedExpressionManager);
+                return new ProgramEvent(event);
             });
         }
 
         this.stackManager.update()
             .progress(this.stackFrameChanged.bind(this))
-            .done( done_after_stack_update.bind(this, response.next_statement) );
+            .done( done_after_stack_update.bind(this) );
 
         events.forEach(function(event) {
             event.render($elt)
@@ -428,7 +374,7 @@ function Debugger(sel) {
         var also = $elt.attr('data-part-of');
         if (also) {
             // hilite it, too
-            $elt.siblings('[data-eval="'+also+'"]').addClass('hovered-perl-var');
+            $elt.parent().find('[data-eval="'+also+'"]').addClass('hovered-perl-var');
         }
         $perlVarPopover = $elt;
 
@@ -439,33 +385,18 @@ function Debugger(sel) {
                 restInterface.getVarAtLevel(eval, stack_level)
                              .done(function(data) { drawPerlPopover(data, $elt) });
             }
-        }, perlVarPopoverDelay);
+        }, 400);
         $perlVarPopover.data('timerid', timer);
     }
 
     function removePopoverPerlVar (e) {
-        var $target = $(e.target),
-            part_of = $target.attr('data-part-of');
-
-        if (part_of !== undefined) {
-            $target = $target.add('[data-eval="'+part_of+'"]');
-        }
-
-        $target.removeClass('hovered-perl-var');
+        $(e.target).removeClass('hovered-perl-var');
         if ($perlVarPopover) {
             $perlVarPopover.popover('destroy');
             $perlVarPopover = undefined;
         }
     }
 
-    function addPerlVarToWatchExpressions(e) {
-        var $target = $(e.target),
-            expr = $target.attr('data-eval');
-
-        e.preventDefault();
-        this.watchedExpressionManager.addExpression(expr, false);
-    }
-
     var $lastControlKeyElement = $control_buttons.filter('[data-action="stepin"]');
     function setKeybindings() {
         // Seems that divs can't receive key events, so we can't bind the
@@ -527,257 +458,14 @@ function Debugger(sel) {
                     $('#breakpoint-container-handle').click();
                     e.stopPropagation();
                     break;
-                case 66: //B
+                case 98: //b
                     dbg.stackDiv.find('.tab-pane.active .code-line.active span.lineno').click();
                     e.stopPropagation();
                     break;
-                case 98: //b
-                    dbg.quickBreakpointDialog();
-                    e.stopPropagation();
-                    break;
             }
         });
     }
 
-    var quick_bp_dialog_open = false;
-    function breakpoint_and_continue_to_dialog_handler(params) {
-        if (quick_bp_dialog_open) {
-            return;
-        }
-        quick_bp_dialog_open = true;
-
-        var modal = $(this.templates.quickBreakpointModal(params['dialog']));
-
-        modal.appendTo($elt)
-            .modal({ backdrop: true, keyboard: true, show: true })
-            .on('hidden', function() {
-                quick_bp_dialog_open = false;
-                modal.remove();
-            })
-            .keyup(function(e) {
-                if (e.keyCode == 27) { // escape - abort editing
-                    modal.modal('hide');
-                    return false;
-                }
-            })
-            .find('input')
-                .focus();
-
-        modal.find('form')
-            .submit(function(e) {
-                    e.preventDefault();
-                    var bp_text = $(e.target).find('[name=breakpoint]').val();
-
-                    parseQuickBreakpointText(bp_text)
-                        .done(params['done'])
-                        .fail(function(message) {
-                            alert(message);
-                        })
-                        .always(function() {
-                            modal.modal('hide');
-                        });
-            });
-    }
-
-    function QB_isCurrentLine(text) {
-        var d = $.Deferred();
-        if (text.match(/^\.$/)) {  // A single period
-            var current_frameno = $('div#stack-panes .tab-pane.active').attr('data-frameno'),
-                stack_frame = dbg.stackManager.frame(current_frameno);
-            d.resolve(stack_frame.filename, stack_frame.line);
-        } else {
-            d.reject(undefined);
-        }
-        return d;
-    }
-
-    function QB_isLineNumber(text) {
-        var d = $.Deferred();
-        var matches = text.match(/^\d+$/);  // match only digits - a line number in the current stackframe's file
-        if (matches) {
-            var line = dbg.stackDiv.find('.tab-pane.active .code-line[data-lineno='+text+']');
-            if (line.length == 0) {
-                d.reject('No such line number in the current file');
-            } else if (line.hasClass('unbreakable')) {
-                d.reject('Line ' + text + ' is not breakable');
-            } else {
-                var file = dbg.stackDiv.find('.tab-pane.active .program-code-container').attr('data-filename');
-                d.resolve(file, text);
-            }
-        } else {
-            d.reject(undefined);
-        }
-        return d.promise();
-    }
-
-    function QB_isFQSubroutine(text) {
-        var d = $.Deferred();
-        var matches = text.match(/^(.+?::\w+)(?::(\d+))?$/); // match "package::subname" or "package::subname:line"
-        if (matches) {
-            var subname = matches[1],
-                line = matches[2];
-
-            restInterface.subInfo(subname)
-                    .done(function(data) {
-                        _QB_findBreakableLineInSub(line, d, data);
-                    })
-                    .fail(function() {
-                        d.reject(undefined);
-                    })
-
-        } else {
-            d.reject(undefined);
-        }
-        return d.promise();
-    }
-
-    function QB_isSubroutine(text) {
-        var d = $.Deferred();
-        var matches = text.match(/^(.+?)(?::(\d+))?$/);  // match either "string" or "string:numbers"
-        if (matches) {
-            var subname = matches[1],
-                line = matches[2],
-                current_frameno = $('div#stack-panes .tab-pane.active').attr('data-frameno'),
-                stack_frame = dbg.stackManager.frame(current_frameno),
-                current_package = stack_frame.package;
-
-            restInterface.subInfo(current_package + '::' + subname)
-                        .done(function(data) {
-                            _QB_findBreakableLineInSub(line, d, data);
-                        })
-                        .fail(function(data) {
-                            d.reject(undefined);
-                        });
-        } else {
-            d.reject(undefined);
-        }
-        return d.promise();
-    }
-
-    function _QB_findBreakableLineInSub(requested_line_str, deferred, subinfo_data) {
-        var start_breakable_search,
-            sub_start_line = parseInt(subinfo_data.line, 10),
-            sub_end_line = parseInt(subinfo_data.end, 10),
-            requested_line = parseInt(requested_line_str, 10);
-
-        if (requested_line_str) {  // They wanted to stop on a particular line
-            var sub_length = sub_end_line - sub_start_line;
-            if (requested_line > sub_length) {
-                deferred.reject(requested_line_str + ' is outside the range of sub ' + subinfo_data.subroutine);
-            }
-            start_breakable_search = sub_start_line + requested_line;
-
-            if (dbg.fileManager.isBreakable(subinfo_data.filename, sub_start_line)) {
-                // if the sub's first line is breakable, then line "1" of the sub
-                // is the same as the sub's start line
-                start_breakable_search = start_breakable_search - 1;
-            }
-
-        } else {
-            start_breakable_search = sub_start_line;
-        }
-
-        for(var break_on_line = start_breakable_search; break_on_line < subinfo_data.end; break_on_line++) {
-            if (dbg.fileManager.isBreakable(subinfo_data.filename, break_on_line)) {
-                deferred.resolve(subinfo_data.filename, break_on_line);
-            }
-        }
-        deferred.reject("Couldn't file a breakable line within '"
-                        + subinfo_data.subroutine
-                        + '. Tried line ' + requested_line + ' to the end of the function');
-    }
-
-    function QB_isFile(text) {
-        var d = $.Deferred();
-        var matches = text.match(/^(.+?):(\d+)$/);  // match "string:numbers"
-        if (matches) {
-            var filename = matches[1],
-                line = matches[2];
-            if (dbg.fileManager.isLoaded(filename)) {
-                if (dbg.fileManager.isBreakable(filename, line)) {
-                    d.resolve(filename, line);
-                } else {
-                    d.reject(filename + ':' + line + ' is not breakable');
-                }
-            } else {
-                d.reject(undefined);
-            }
-        } else {
-            d.reject(undefined);
-        }
-        return d.promise();
-    }
-
-    function QB_unparsable(text) {
-        var d = $.Deferred();
-        d.reject("Could not parse breakpoint '" + text + "'");
-        return d.promise();
-    }
-
-    var quick_breakpoint_resolvers = [
-        QB_isCurrentLine,
-        QB_isLineNumber,
-        QB_isFQSubroutine,
-        QB_isSubroutine,
-        QB_isFile,
-        QB_unparsable,
-    ];
-    function parseQuickBreakpointText(text) {
-        var d = $.Deferred();
-
-        function tryBreakpointResolver(i) {
-            var resolver = quick_breakpoint_resolvers[i],
-                p = resolver(text);
-
-            p.done(function(filename, line) {
-                d.resolve(filename, line);  // try just putting d in the done()?
-            })
-            .fail(function(message) {
-                if (message) {
-                    d.reject(message);
-                } else {
-                    tryBreakpointResolver(i + 1);
-                }
-            })
-        }
-
-        tryBreakpointResolver(0);
-        return d.promise();
-    }
-
-    this.quickBreakpointDialog = function() {
-        breakpoint_and_continue_to_dialog_handler.call(this, {
-            dialog: {
-                title: 'Quick-add Breakpoint',
-                prompt: 'New breakpoint',
-                ok_button: 'Add'
-            },
-            done: function(filename, line) {
-                dbg.breakpointManager.createBreakpoint({filename: filename, line: line});
-            }
-        });
-    };
-
-    this.continueToDialog = function() {
-        breakpoint_and_continue_to_dialog_handler.call(this, {
-            dialog: {
-                title: 'Continue To...',
-                prompt: 'Continue to',
-                ok_button: 'Continue'
-            },
-            done: function(filename, line) {
-                restInterface.createBreakpoint({
-                    filename: filename,
-                    line: line,
-                    once: 1,
-                    code: '1'
-                });
-                $control_buttons.filter('[data-action="continue"]')
-                                        .click();
-            }
-        });
-    };
-                                    
     var $breakpointPane = $('#breakpoint-container'),
         $breakpointToggleIcon = $('#breakpoint-container-handle-icon');
 
@@ -934,28 +622,12 @@ function Debugger(sel) {
         $filePane.remove();
     }
 
-    function _loadConfig_response_handler_for_saveLoadBreakpoints(settings) {
-        dbg.breakpointManager.sync();
-
-        if (settings['additional'] && settings['additional']['watch_expressions']) {
-            var watch_expressions = settings['additional']['watch_expressions'];
-            for (var i = 0; i < watch_expressions.length; i++) {
-                dbg.watchedExpressionManager.addExpression( watch_expressions[i], false);
-            }
-        }
-    }
-
-    this.defaultConfigFileName = function() {
-        return this.programName + '.hdb';
-    };
-
     function saveLoadBreakpoints(e) {
         e.preventDefault();
         var isSave = $(e.currentTarget).attr('id') === 'save-breakpoints',
-            savefile = dbg.defaultConfigFileName(),
             modal = $(this.templates.saveLoadBreakpointsModal({
                                             action: isSave ? 'Save' : 'Load',
-                                            filename: savefile
+                                            filename: this.programName + '.hdb'
                                         }));
         modal.appendTo($elt)
              .modal({ backdrop: true, keyboard: true, show: true })
@@ -965,22 +637,17 @@ function Debugger(sel) {
         modal.find('form')
              .submit(function(e) {
                     e.preventDefault();
-                    var ri_method = restInterface[isSave ? 'saveConfig' : 'loadConfig'];
-                        additional = {};
+                    var savefile = $(e.target).find('[name=filename]').val(),
+                        ri_method = restInterface[isSave ? 'saveConfig' : 'loadConfig'];
 
-                    if (isSave) {
-                        additional['watch_expressions'] = dbg.watchedExpressionManager.expressions();
-                        console.log(additional);
-                    }
-
-                    ri_method.call(restInterface, savefile, additional)
+                    ri_method.call(restInterface, savefile)
                              .fail(function(jqxhr, text_status, error_thrown) {
                                 alert((isSave ? 'Saving' : 'Loading')
                                         + ' config failed: ' + error_thrown);
                                 })
-                            .done(function(settings) {
+                            .done(function() {
                                  if (! isSave) {
-                                    _loadConfig_response_handler_for_saveLoadBreakpoints(settings)
+                                    dbg.breakpointManager.sync();
                                 }
                             })
                             .always(function() {
@@ -992,41 +659,26 @@ function Debugger(sel) {
     // Initialization
     this.restInterface.programName()
         .done(function(program_name) {
-            $elt.find('li#program-name a').html(program_name)
-                                          .attr('title', program_name);
+            $elt.find('li#program-name a').html(program_name);
             dbg.programName = program_name;
-        })
-        .done(function() {
-            // This has to happen after the above where it sets the programName
-            dbg.restInterface.loadConfig(dbg.defaultConfigFileName())
-                            .done(function(settings) {
-                                _loadConfig_response_handler_for_saveLoadBreakpoints(settings);
-                            });
         });
     this.fileManager = new FileManager(this.restInterface);
 
     this.breakpointManager = new BreakpointManager(this.$elt, this.restInterface);
     this.watchedExpressionManager = new WatchedExprManager(this.restInterface);
-    this.watchedExpressionManager.loadWatchpoints();
 
     this.stackManager = new StackManager(this.restInterface);
     this.stackManager.initialize()
         .progress(this.stackFrameChanged.bind(this))
-        .done(function(status) {
+        .done(function() {
                 done_after_stack_update.call(dbg);
-                dbg.setCurrentStatementForCodeTable(status.next_statement);
+                dbg.breakpointManager.sync();
         });
 
     // Events
-    $elt.on('click', '.control-button[name="stepin"][disabled!="disabled"]',    this.controlButtonClicked.bind(this))
-        .on('click', '.control-button[name="stepout"][disabled!="disabled"]',   this.controlButtonClicked.bind(this))
-        .on('click', '.control-button[name="stepover"][disabled!="disabled"]',  this.controlButtonClicked.bind(this))
-        .on('click', '.control-button[name="continue"][disabled!="disabled"]',  this.controlButtonClicked.bind(this))
-        .on('click', '.control-button[name="exit"][disabled!="disabled"]',      this.controlButtonClicked.bind(this))
-        .on('click', '.control-button[name="continue-to"][disabled!="disabled"]', this.continueToDialog.bind(this))
+    $elt.on('click', '.control-button[disabled!="disabled"]', this.controlButtonClicked.bind(this))
         .on('mouseenter', '.popup-perl-var', popoverPerlVar.bind(this))
         .on('mouseleave', '.popup-perl-var', removePopoverPerlVar.bind(this))
-        .on('contextmenu', '.popup-perl-var', addPerlVarToWatchExpressions.bind(this))
         .on('click', '.current-sub-and-args', setCurrentLineForCodeTableEvent.bind(this))
         .on('hangup', notifyProgramHasCompletelyExited.bind(this))
         .on('click', '#add-file', pickFileToLoad.bind(this))
