@@ -1,12 +1,6 @@
 function FileManager(rest_interface) {
     var files = {},
-        is_breakable = {},
-        programListingTemplate = Handlebars.compile( $('#program-listing-template').html() ),
-        array_or_hash_element_or_slice = /^(([$@])(\w+(::\w+)*)([[{]))/,
-        scalar_array_hash_or_glob = /^(([$@%*])(\w+(::\w+)*))/,
-        basic_dereference = /^(([@%*])(\$\w+(::\w+)*))/,
-        perl_special_var = /^(([$@%])(?!&(#x[0-9A-Fa-f]{1,4}|quot|amp|apos|lt|gt);)(\^.|.))/,
-        other_perl_special_var = /^(\$&(quot|amp|apos|lt|gt|#x27|#x60);)/;
+        programListingTemplate = Handlebars.compile( $('#program-listing-template').html() );
 
     var markupPerlVars = function(line) {
         var i = 0,  // index into the string
@@ -14,15 +8,15 @@ function FileManager(rest_interface) {
 
         while (i < line.length) {
             var remaining = line.substr(i);
-            if (array_or_hash_element_or_slice.test(remaining)) {
-                var varPrefix = RegExp.$1,
-                    sigil = RegExp.$2,
-                    isSlice = sigil === '@',
-                    varName = RegExp.$3,
-                    openParen = RegExp.$5,
-                    isArray = openParen === '[',
-                    containerSigil = isArray ? '@' : '%',
-                    endpos = remaining.indexOf(isArray ? ']' : '}');
+            if (/^(([$@])(\w+(::\w+)*)([[{]))/.test(remaining)) {
+                // Array or hash element or slice
+                var varPrefix = RegExp.$1;
+                var sigil = RegExp.$2;
+                var isSlice = sigil === '@';
+                var varName = RegExp.$3;
+                var openParen = RegExp.$5;
+                var isArray = openParen === '[';
+                var endpos = remaining.indexOf(isArray ? ']' : '}');
                 if (endpos === null) {
                     // Didn't find it - bail out
                     output = varPrefix;
@@ -36,26 +30,18 @@ function FileManager(rest_interface) {
                             + (isArray ? '@' : '%') + varName + '">' + sigil + varName
                             + '</span><span class="popup-perl-var" data-eval="'
                             + remaining.substring(0, endpos + 1) + '" data-part-of="'
-                            + containerSigil + varName + '">'
+                            + sigil + varName + '">'
                             + arrayIndex + '</span>';
                 i += (endpos + 1);
 
-            } else if (basic_dereference.test(remaining)) {
-                output += '<span class="popup-perl-var" data-eval="'
-                            + RegExp.$3 + '">' + RegExp.$1 + '</span>';
-                i += RegExp.$1.length;
-
-            } else if (scalar_array_hash_or_glob.test(remaining)) {
+            } else if (/^(([$@%*])(\w+(::\w+)*))/.test(remaining)) {
+                // whole scalar, array, hash or glob
                 output += '<span class="popup-perl-var" data-eval="'
                             + RegExp.$1 + '">' + RegExp.$1 + '</span>';
                 i += RegExp.$1.length;
 
-            } else if (perl_special_var.test(remaining)) {
-                output += '<span class="popup-perl-var" data-eval="'
-                            + RegExp.$1 + '">' + RegExp.$1 + '</span>';
-                i += RegExp.$1.length;
-
-            } else if (other_perl_special_var.test(remaining)) {
+            } else if (/^(([$@%])(\^.|.))/.test(remaining)) {
+                // A perl special var
                 output += '<span class="popup-perl-var" data-eval="'
                             + RegExp.$1 + '">' + RegExp.$1 + '</span>';
                 i += RegExp.$1.length;
@@ -76,14 +62,12 @@ function FileManager(rest_interface) {
         if (source_lines) {
             // File is loaded
             templateData = { filename: filename, rows: [] };
-            is_breakable[filename] = {};
             for (i = 0; i < source_lines.length; i++) {
                 templateData.rows.push({
                         line: i+1,
-                        code: Handlebars.Utils.escapeExpression(source_lines[i][0]),  // This gets re-htmlified below in codeElt.html()
+                        code: Handlebars.Utils.escapeExpression(source_lines[i][0]),  // why? The template uses {{code}} which should be auto-escaped!?
                         unbreakable: !source_lines[i][1]
                 });
-                is_breakable[filename][i+1] = (source_lines[i][1] ? true : false);
             }
             var $elt = $( programListingTemplate(templateData) );
             $elt.find('.code').each(function(idx,codeElt) {
@@ -109,11 +93,4 @@ function FileManager(rest_interface) {
         }
         return files[filename].promise()
     };
-
-    this.isLoaded = function(filename) {
-         return ((filename in files) && files[filename]);
-    };
-    this.isBreakable = function(filename, line) {
-        return is_breakable[filename][line];
-    };
 }
