@@ -1,14 +1,6 @@
 function RestInterface(base_url) {
     var rest_interface = this;
 
-    this.status = function(params) {
-        var url = 'status';
-        if (params.next_statement) {
-            url = url + '?next_statement=1';
-        }
-        return this._GET(url, params);
-    };
-
     this.stack = function() {
         return this._GET('stack', undefined);
     };
@@ -50,7 +42,7 @@ function RestInterface(base_url) {
     };
 
     this.fileSourceAndBreakable = function(filename) {
-        return this._GET('source/' + encodeURIComponent(filename), undefined);
+        return this._GET('source/' + filename, undefined);
     };
 
     this.exit = function() {
@@ -103,37 +95,22 @@ function RestInterface(base_url) {
         return this._POST('loadconfig/' + filename);
     };
 
-    this.saveConfig = function(filename, additional) {
-        return this._POST('saveconfig/' + filename, additional);
+    this.saveConfig = function(filename) {
+        return this._POST('saveconfig/' + filename);
     };
 
     ['stepin','stepout','stepover','continue','exit'].forEach(function(action) {
         this[action] = function() {
-            return this._POST(action + '?next_statement=1', undefined);
+            return this._POST(action, undefined);
         };
     }, this);
 
-    this.getWatchpoints = function() {
-        return this._GET('watchpoints');
-    };
-
-    this.createWatchpoint = function(expr) {
-        return this._PUT('watchpoints/' + expr);
-    };
-
-    this.deleteWatchpoint = function(expr) {
-        return this._DELETE('watchpoints/' + expr);
-    };
-
     this._GET = function(url, params) {
         return this._http_request('GET', url, params);
     };
     this._POST = function(url, params) {
         return this._http_request('POST', url, params);
     };
-    this._PUT = function(url, params) {
-        return this._http_request('PUT', url, params);
-    };
     this._DELETE = function(url, params) {
         return this._http_request('DELETE', url, params);
     };
